import { route } from 'quasar/wrappers';
import { useAppStore } from 'stores/app';
import {
    createRouter,
    createMemoryHistory,
    createWebHistory,
    createWebHashHistory
} from 'vue-router';
import routes from './routes';
import { i18n } from 'src/boot/i18n';

/*
 * If not building with SSR mode, you can
 * directly export the Router instantiation;
 *
 * The function below can be async too; either use
 * async/await or return a Promise which resolves
 * with the Router instance.
 */

export default route(function (/* { store, ssrContext } */) {
    const createHistory = process.env.SERVER
        ? createMemoryHistory
        : process.env.VUE_ROUTER_MODE === 'history'
          ? createWebHistory
          : createWebHashHistory;

    const Router = createRouter({
        scrollBehavior: () => ({ left: 0, top: 0 }),
        routes,

        // Leave this as is and make changes in quasar.conf.js instead!
        // quasar.conf.js -> build -> vueRouterMode
        // quasar.conf.js -> build -> publicPath
        history: createHistory(
            process.env.MODE === 'ssr' ? void 0 : process.env.VUE_ROUTER_BASE
        )
    });

    Router.afterEach((to, from) => {
        if (from.name === to.name) return;
        const app = useAppStore();
        app.$patch({
            title: i18n.global.t(
                to.meta.title ? `titles.${to.meta.title}` : 'titles.Home'
            ),
            subtitle: null,
            useRightDrawer: false,
            rightDrawerOpen: true
        });
    });

    return Router;
});
