const routes = [
    {
        path: '/login',
        component: () => import('layouts/LoginLayout.vue'),
        children: [
            {
                name: 'login',
                path: '/login/:email?',
                component: () => import('pages/Login/LoginView.vue')
            },
            {
                name: 'rememberPassword',
                path: '/remember-password',
                component: () => import('pages/Login/RememberPassword.vue')
            },
            {
                name: 'resetPassword',
                path: '/reset-password',
                component: () => import('pages/Login/ResetPassword.vue')
            }
        ]
    },
    {
        path: '/',
        component: () => import('layouts/MainLayout.vue'),
        children: [
            {
                name: '',
                path: '',
                component: () => import('src/pages/Cms/HomeView.vue')
            },
            {
                name: 'home',
                path: '/cms/home',
                meta: {
                    title: 'Home'
                },
                component: () => import('src/pages/Cms/HomeView.vue')
            },
            {
                name: 'confirmedOrders',
                path: '/ecomerce/orders',
                meta: {
                    title: 'Last orders'
                },
                component: () => import('pages/Ecomerce/OrdersView.vue')
            },
            {
                name: 'ticket',
                path: '/ecomerce/ticket/:id',
                meta: {
                    title: 'Ticket'
                },
                component: () => import('pages/Ecomerce/TicketView.vue')
            },
            {
                name: 'invoices',
                path: '/ecomerce/invoices',
                meta: {
                    title: 'Invoices'
                },
                component: () => import('pages/Ecomerce/InvoicesView.vue')
            },
            {
                name: 'pendingOrders',
                path: '/ecomerce/pending',
                meta: {
                    title: 'Pending orders'
                },
                component: () => import('pages/Ecomerce/PendingOrders.vue')
            },
            {
                name: 'catalog',
                path: '/ecomerce/catalog/:category?/:type?',
                meta: {
                    title: 'Catalog'
                },
                component: () => import('pages/Ecomerce/CatalogView.vue')
            },
            {
                name: 'basket',
                path: '/ecomerce/basket/:id?',
                meta: {
                    title: 'Basket'
                },
                component: () => import('pages/Ecomerce/BasketView.vue')
            },
            {
                name: 'confirm',
                path: '/ecomerce/confirm/:id?',
                meta: {
                    title: 'OrderSummary'
                },
                component: () => import('pages/Ecomerce/ConfirmView.vue')
            },
            {
                name: 'checkout',
                path: '/ecomerce/checkout/:id?',
                meta: {
                    title: 'Checkout'
                },
                component: () => import('pages/Ecomerce/CheckoutView.vue')
            },
            {
                name: 'agencyPackages',
                path: '/agencies/packages',
                meta: {
                    title: 'Agencies'
                },
                component: () => import('src/pages/Agencies/PackagesView.vue')
            },
            {
                name: 'accountConfig',
                path: '/account/conf',
                meta: {
                    title: 'Account'
                },
                component: () => import('pages/Account/AccountConfig.vue')
            },
            {
                name: 'addressesList',
                path: '/account/address-list',
                meta: {
                    title: 'Addresses'
                },
                component: () => import('pages/Account/AddressList.vue')
            },
            {
                name: 'addressDetails',
                path: '/account/address/:id?',
                meta: {
                    title: 'Address details'
                },
                component: () => import('pages/Account/AddressDetails.vue')
            },
            {
                name: 'controlPanel',
                path: 'admin/links',
                meta: {
                    title: 'Control panel'
                },
                component: () => import('pages/Admin/LinksView.vue')
            },
            {
                name: 'adminUsers',
                path: 'admin/users:/:id?',
                meta: {
                    title: 'Users'
                },
                component: () => import('pages/Admin/UsersView.vue')
            },
            {
                name: 'accessLog',
                path: 'admin/access-log/:id?',
                meta: {
                    title: 'Access log'
                },
                component: () => import('pages/Admin/AccessLogView.vue')
            },
            {
                name: 'adminConnections',
                path: 'admin/connections',
                meta: {
                    title: 'Connections'
                },
                component: () => import('pages/Admin/ConnectionsView.vue')
            },
            {
                name: 'adminVisits',
                path: 'admin/visits',
                meta: {
                    title: 'Visits'
                },
                component: () => import('pages/Admin/VisitsView.vue')
            },
            {
                name: 'adminNews',
                path: 'news/news',
                meta: {
                    title: 'News'
                },
                component: () => import('pages/Admin/NewsView.vue')
            },
            {
                name: 'adminNewsDetails',
                path: 'news/new/:id?',
                meta: {
                    title: 'Admin news details'
                },
                component: () => import('pages/Admin/NewsDetails.vue')
            },
            {
                name: 'adminPhotos',
                path: 'admin/photos',
                meta: {
                    title: 'Photos'
                },
                component: () => import('pages/Admin/PhotosView.vue')
            },
            {
                name: 'adminItems',
                path: 'admin/items',
                meta: {
                    title: 'Items'
                },
                component: () => import('pages/Admin/ItemsView.vue')
            }
        ]
    },

    // Always leave this as last one,
    // but you can also remove it
    {
        path: '/:catchAll(.*)*',
        component: () => import('pages/ErrorNotFound.vue')
    }
];

export default routes;
