import { defineStore } from 'pinia';
import { jApi } from 'boot/axios';

export const tpvStore = defineStore('tpv', {
    actions: {
        async check(route) {
            const order = route.query.tpvOrder;
            const status = route.query.tpvStatus;
            if (!(order && status)) return null;

            await jApi.execQuery('CALL myTpvTransaction_end(#order, #status)', {
                order,
                status
            });

            if (status === 'ko') {
                const retry = confirm('retryPayQuestion');
                if (retry) {
                    this.retryPay(order);
                }
            }

            return status;
        },

        async pay(amount, company) {
            await this.realPay(amount * 100, company);
        },

        async retryPay(order) {
            const payment = await jApi.getObject(
                `SELECT t.amount, m.companyFk
          FROM myTpvTransaction t
            JOIN tpvMerchant m ON m.id = t.merchantFk
          WHERE t.id = #order`,
                { order }
            );
            await this.realPay(payment.amount, payment.companyFk);
        },

        async realPay(amount, company) {
            if (!isNumeric(amount) || amount <= 0) {
                throw new Error('payAmountError');
            }

            const json = await jApi.send('tpv/transaction', {
                amount: parseInt(amount),
                urlOk: this.makeUrl('ok'),
                urlKo: this.makeUrl('ko'),
                company
            });

            const postValues = json.postValues;

            const form = document.createElement('form');
            form.method = 'POST';
            form.action = json.url;
            document.body.appendChild(form);

            for (const field in postValues) {
                const input = document.createElement('input');
                input.type = 'hidden';
                input.name = field;
                form.appendChild(input);

                if (postValues[field]) {
                    input.value = postValues[field];
                }
            }

            form.submit();
        },

        makeUrl(status) {
            let path = location.protocol + '//' + location.hostname;
            path += location.port ? ':' + location.port : '';
            path += location.pathname;
            path += location.search ? location.search : '';
            path += '#/ecomerce/orders';
            path +=
                '?' +
                new URLSearchParams({
                    tpvStatus: status,
                    tpvOrder: '_transactionId_'
                }).toString();
            return path;
        }
    }
});

function isNumeric(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}
